Running flex-api-server under uwsgi for local development
------------------------------------------------

Note that flex-api-server is only supported under a Debian-based Linux
distribution.

1. Install uWSGI, `sudo apt-get install uwsgi`
    1. An example uWSGI config file is shipped in the `examples/uwsgi/`
        directory.  `localdev_dnstli2.ini` runs the server with API key
        authentication and uses uWSGI's built-in HTTP server to listen on
        `127.0.0.1`.
2. Add [the Farsight apt package repository](https://www.farsightsecurity.com/technical/SIE-user-guide/sie-debian/)
    if it is not added already
3. Install the required packages in the `debian/control` file
4. python3-dnstable and python3-wdns might not install on Buster because they require python3.6
    but Buster comes with python3.7
    1. In this case, clone the source repos: [pydnstable](https://github.com/farsightsec/pydnstable)
        and [pywdns](https://github.com/farsightsec/pywdns)
    2. Install the required packages in their `debian/control` files
    3. Run `sudo python3 setup.py install` for each
5. Install redis with `sudo apt-get install redis`
    1. You can test your redis install with `redis-cli ping`
    2. The response should be "PONG"
6. Add a `/etc/dnstli/flexapiserver.conf` file, you can copy `examples/flexapiserver.conf`
    1. Make sure to change the "query_limit_redis_url" field to the correct value for your local redis install
    2. For example:
    ```
    "query_limit_redis_url": "redis://127.0.0.1:6379"
    ```
7. Add a `/srv/tgdb/rrnames.fileset` and some tgdb files
    1. You can grab tgdb data from a stage server like stage-dnsdb-query-1.iad1.fsi.io
    2. Add the names of the tgdb files you copied to `/srv/tgdb/rrnames.fileset`
8. Clone and install flex-backend
    1. git clone [the flex-backend source code](https://bitbucket.it.fsi.io/projects/DNSDB/repos/flex-backend/browse)
    2. Install all the dependencies in the `debian/control` file
    3. Run the `./autogen.sh` command first to generate the `./configure` script
    4. Then `./configure && make && sudo make install`
    5. Create a `/run/flexbackendd/` directory if it doesn't already exist, make sure it has the right permissions
    5. Start flex-backend:
        ```
        flexbackendd -vv -F rrnames:/srv/tgdb/rrnames.fileset -u /run/flexbackendd/flex_api_socket -U 777 -t 900 -T 900 -H 60
        ```
    6. You can read more about flex-backend utilities and continous processing of dnstable files to tgdb files on
        the [flex-backend README](https://bitbucket.it.fsi.io/projects/DNSDB/repos/flex-backend/browse/README.md)
8. Add an api key file to `/etc/dnstli/auth-api`
    1. You can again copy your api key file from a stage server like
        stage-dnsdb-query-1.iad1.fsi.io
9. Install the flexapiserver uwsgi module
    ```
    sudo python3 ./setup.py build install
    ```
9. Run the localdev API server, invoke the following command from the top
    directory of the `flex-api-server` working tree:
    ```
    uwsgi --http-socket 127.0.0.1:65081 --plugin syslog --plugin python3 --ini examples/uwsgi/localdev_flexapiserver.ini
    ```
10. Test a query with `curl -g -H "Accept: application/x-ndjson" -H "X-API-Key: $DNSDB_API_KEY" 'http://localhost:65081/dnsdb/v2/regex/rrnames/^fsi\.io/ANY?limit=2'`
    1. Alternatively you can use the `dnsdbq` command when the `DNSDB_SERVER`
        variable is set to `"127.0.0.1:65081"`
    2. Make sure to set the DNSDB_API_KEY env variable to the file name you
        added in `/etc/dnstli/auth-api`
11. To get actual results, look into the tgdb file using `flexsearch`
    1. `flexsearch` was built as part of flex-backend's make above
    2. To see a few rrnames in the tgdb file run
    ```
    flexsearch -l /srv/tgdb/your-tgdb-file.tgdb -s '.*' | head
    ```
    3. If you are not getting any results even with a name in the tgdb file, make sure the tgdb fileset and tgdb files have the right permissions

The localdev uWSGI config files are configured to send debugging output related
to the application server itself to `stderr`, while the flex-api-server application will
send its own log data to syslog.

For a complete guide to installing on Debian, see the "DNSDB Export Installation for Debian Stretch" document under [Restricted Documentation - DNSDB Export](https://domaintools.atlassian.net/wiki/spaces/SharedServices/pages/3642397379/FSI+Documentation+Repository#DNSDB-Export.1).
